@@ -3,6 +3,9 @@
  * Copyright (c) 1998-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: alarm.c,v $
+ * Revision 1.2  2000/02/10 23:49:17  jab
+ * fixed reporting of the year the alarm dates are set to
+ *
  * Revision 1.1  1999/12/04 13:40:07  jab
  * Initial revision
  *
@@ -23,7 +26,7 @@
  *
  */
 
-static char const rcsid[]="$Id: alarm.c,v 1.1 1999/12/04 13:40:07 jab Exp jab $";
+static char const rcsid[]="$Id: alarm.c,v 1.2 2000/02/10 23:49:17 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -39,8 +42,8 @@ static char const rcsid[]="$Id: alarm.c,v 1.1 1999/12/04 13:40:07 jab Exp jab $"
 
 
 #define VERSION_STRING "\
-Alarm version 1.0\n\
-Copyright (c) 1998-99 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
+Alarm version 1.1\n\
+Copyright (c) 1998-2000 Jonathan A. Buzzard <jonathan@buzzard.org.uk>\n"
 
 #define USAGE_STRING "\
 Usage: alarm [-d] [-l] [-h] [-v] time\n\
@@ -81,6 +84,7 @@ int AlarmSupport(void)
 
 	reg.ebx = SCI_ALARM_DATE;
 	if (SciGet(&reg)==SCI_SUCCESS) {
+		reg.edx &= 0xffff;
 		support |= ALARM_DATE;
 		if ((reg.edx & SCI_ALARM_EVERYDAY)==0x0001)
 			support |= ALARM_EVERY;
@@ -94,6 +98,22 @@ int AlarmSupport(void)
 }
 
 
+/*
+ * Is time one before time two?
+ */
+int Behind(struct tm *one, struct tm *two)
+{	
+	if ((one->tm_mon<=two->tm_mon) && (one->tm_mday<=two->tm_mday)
+		&& (one->tm_hour<=two->tm_hour) && (one->tm_sec<two->tm_sec)) {
+
+		return 1;
+	}
+
+	return 0;
+}
+
+
+
 int main(int argc, char *argv[])
 {
 	SMMRegisters reg;
@@ -101,6 +121,7 @@ int main(int argc, char *argv[])
 	int i,version,local,disable,support;
 	char buffer[64];
 	struct tm *alarm;
+	struct tm current;
 
 
 	/* do some quick checks on the laptop */
@@ -197,12 +218,25 @@ int main(int argc, char *argv[])
 			reg.ecx = SCI_FULLDATE((alarm->tm_year-90),
 				(1+alarm->tm_mon), alarm->tm_mday);
 		else
-			reg.ecx = SCI_DATE((1+alarm->tm_mon),
-				alarm->tm_mday);
-		i = SciSet(&reg);
+			reg.ecx = SCI_DATE((1+alarm->tm_mon), alarm->tm_mday);
+
+		SciSet(&reg);
 		SciCloseInterface();
 	}
 
+	/* save the current time and date */
+
+	time(&timer);
+	if (local==1)
+		alarm = localtime(&timer);
+	else
+		alarm = gmtime(&timer);
+
+	current.tm_year = alarm->tm_year;
+	current.tm_mon = alarm->tm_mon;
+	current.tm_mday = alarm->tm_mday;
+	current.tm_hour = alarm->tm_hour;
+	current.tm_min = alarm->tm_min;
 
 	/* display the current alarm power on setting */
 
@@ -228,7 +262,10 @@ int main(int argc, char *argv[])
 		if ((reg.ecx & SCI_ALARM_EVERYDAY)==0x0001)
 			strftime(buffer, 64, "%R everyday", alarm);
 		if (alarm->tm_year==1990) {
-			alarm->tm_year = 1999-1900;
+			if (Behind(alarm, &current)==1)
+				alarm->tm_year = 1+current.tm_year;
+			else
+				alarm->tm_year = current.tm_year;
 			mktime(alarm);
 			strftime(buffer, 64, "%a %b %e %R %Y", alarm);
 		} else {
