@@ -3,6 +3,9 @@
  * Copyright (c) 1997-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: hotkey.c,v $
+ * Revision 2.3  2000/02/12 21:35:30  jab
+ * fix so Stockholm class models display economy power graphic
+ *
  * Revision 2.2  2000/01/15 10:34:57  jab
  * changed Fn detection to new /proc interface
  * fixed a whole slew of bugs so it actually works
@@ -57,7 +60,7 @@
  *
  */
 
-static char const rcsid[]="$Id: hotkey.c,v 2.2 2000/01/15 10:34:57 jab Exp jab $";
+static char const rcsid[]="$Id: hotkey.c,v 2.3 2000/02/12 21:35:30 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -69,11 +72,7 @@ static char const rcsid[]="$Id: hotkey.c,v 2.2 2000/01/15 10:34:57 jab Exp jab $
 #include<signal.h>
 #include<sys/types.h>
 #include<paths.h>
-
-/* this deals with the differences between libc5 and glib2 */
-#ifdef __GLIBC__
-#include<sys/perm.h>
-#else
+#ifndef __GLIBC__
 #define X_LOCALE
 #endif
 
@@ -213,7 +212,7 @@ void DestroyBox(void)
 void PaintBox(int style)
 {
 	XWindowAttributes attrib;
-	int x,y,width,height;
+	int p,x,y,width,height;
 
 	XGetWindowAttributes(display, box, &attrib);
 	width = attrib.width;
@@ -238,17 +237,29 @@ void PaintBox(int style)
 					XDrawPoint(display, box, gc, 23+x, 25+y);
 				}
 		for(y=0;y<32;y++)
-			for(x=0;x<32;x++)
-				if (lowpower[y][x]!=0x20) {
-					XSetForeground(display, gc, pixels[lowpower[y][x]-0x30]);
+			for(x=0;x<32;x++) {
+				p = (type==STOCKHOLM) ? economy[y][x] : lowpower[y][x];
+				if (p!=0x20) {
+					XSetForeground(display, gc, pixels[p-0x30]);
 					XDrawPoint(display, box, gc, 84+x, 25+y);
 				}
+			}
 		for(y=0;y<32;y++)
-			for(x=0;x<32;x++)
-				if (userpower[y][x]!=0x20) {
-					XSetForeground(display, gc, pixels[userpower[y][x]-0x30]);
+			for(x=0;x<32;x++) {
+				p = (type==STOCKHOLM) ? lowpower[y][x] : userpower[y][x];
+				if (p!=0x20) {
+					XSetForeground(display, gc, pixels[p-0x30]);
 					XDrawPoint(display, box, gc, 145+x, 25+y);
 				}
+			}
+		if (type==STOCKHOLM) {
+			for(y=0;y<32;y++)
+				for(x=0;x<32;x++)
+					if (userpower[y][x]!=0x20) {
+						XSetForeground(display, gc, pixels[userpower[y][x]-0x30]);
+						XDrawPoint(display, box, gc, 206+x, 25+y);
+					}
+		}
 	}
 
 	if (style==0x3d) {
