@@ -3,6 +3,10 @@
  * Copyright (c) 1996-98  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: machine.c,v $
+ * Revision 1.3  2000/02/11 00:00:23  jab
+ * fixed ambiguously nested if-elses
+ * super bright LCD settings fixed courtesy of Tom May <tom@you-bastards.com>
+ *
  * Revision 1.2  1999/12/04 13:43:21  jab
  * changed for the new tuxtime-conf program
  *
@@ -26,7 +30,7 @@
  *
  */
 
-static const char rcsid[]="$Id: machine.c,v 1.2 1999/12/04 13:43:21 jab Exp jab $";
+static const char rcsid[]="$Id: machine.c,v 1.3 2000/02/11 00:00:23 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -253,7 +257,7 @@ static MenuItem lcdbrightmenu[] =
 
 static MenuItem lcdsupermenu[] =
 {
-	{"Super-Bright", LCDCallback, (gpointer) SCI_BRIGHT, NULL},
+	{"Super-Bright", LCDCallback, (gpointer) SCI_SUPER_BRIGHT, NULL},
 	{"Bright", LCDCallback, (gpointer) SCI_BRIGHT, NULL},
 	{"Semi-Bright", LCDCallback, (gpointer) SCI_SEMI_BRIGHT, NULL},
 	{NULL, NULL, NULL, NULL}
@@ -435,15 +439,16 @@ void SetBatteryOptions(int mode)
 	if ((i = GetMenuItemNumber(sleepmenu, settings.sleep))!=-1)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(sleep), i);
 
-	if (lcdtype==BRIGHT)
+	if (lcdtype==BRIGHT) {
 		if ((i = GetMenuItemNumber(lcdbrightmenu, settings.lcd))!=-1)
 			gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), i);
-	else if (lcdtype==SUPER)
+	} else if (lcdtype==SUPER) {
 		if ((i = GetMenuItemNumber(lcdsupermenu, settings.lcd))!=-1)
 			gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), i);
-	else
+	} else {
 		if ((i = GetMenuItemNumber(lcdbackmenu, settings.lcd))!=-1)
 			gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), i);
+	}
 
 	if ((i = GetMenuItemNumber(coolingmenu, settings.cooling))!=-1)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(cooling), i);
@@ -817,7 +822,7 @@ void MachineOkCallback(GtkWidget *widget, GtkWidget *dialog)
 
 	/* exit deleting pid file */
 
-	ExitTuxTimeConfig(200);
+	ExitTuxTimeConfig(0x20);
 
 	return;
 }
@@ -1479,11 +1484,12 @@ void HourCallback(GtkWidget *widget, gpointer data)
 	}
 	else {
 		hours--;
-		if (hours==-1)
+		if (hours==-1) {
 			if (alarmformat==3)
 				hours = 23;
 			else
 				hours = 12;
+		}
 	}
 
 	sprintf(buffer, "%02d:%02d", hours, minutes);
