@@ -407,12 +407,14 @@ int ConvertTime(char *time, int *hour, int *minute)
 
 	/* deal with any am or pm switch */
 
-	if (*(time+2)!='\0')
-		if ((*(time+2)=='p') || (*(time+2)=='P'))
+	if (*(time+2)!='\0') {
+		if ((*(time+2)=='p') || (*(time+2)=='P')) {
 			*hour+=12;
-		else if ((*(time+2)!='h') && (*(time+2)!='H') &&
-			(*(time+2)!='a') && (*(time+2)!='A'))
+		} else if ((*(time+2)!='h') && (*(time+2)!='H') &&
+			(*(time+2)!='a') && (*(time+2)!='A')) {
 			return 0;
+		}
+	}
 
 	/* check to see if the converted time is valid */
 
