@@ -1,26 +1,26 @@
 /* ownerstring.c -- view/change the owner string on a Toshiba laptop.
  *
- * Copyright (c) 1998/99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
+ * Copyright (c) 1998-2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log:$
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 675 Mass Ave, Cambridge, MA 02139, USA.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
-static const char rcsid[]="";
+static const char rcsid[]="$Id:$";
 
 #include<stdio.h>
 #include<stdlib.h>
@@ -31,8 +31,6 @@ static const char rcsid[]="";
 #endif
 #include<signal.h>
 #include<paths.h>
-#include<readline/readline.h>
-#include<readline/history.h>
 
 #include"sci.h"
 #include"hci.h"
@@ -50,7 +48,7 @@ Usage: ownerstring [-h|v]\n\
 /*
  * Catch any signals and exit the program in nicely.
  */
-void catch(int x)
+void Catch(int x)
 {
 	fprintf(stderr, "ownerstring: caught signal %d, cleaning up...\n", x);
 	unlink(PID_FILE);
@@ -78,36 +76,46 @@ char *GetOwnerString(void)
 {
 	SMMRegisters reg;
 	char *owner,*string;
-	long characters;
+	unsigned long characters;
 	int i,j,c,length,valid;
 
 	/* how big is the current owner string */
 
 	reg.eax = HCI_GET;
+	reg.ebx = HCI_OWNERSTRING;
 	reg.esi = 0x0000;
 	HciFunction(&reg);
 	length = (reg.ecx & 0xffff0000)>>16;
+	printf("length=%d\n", length);
 	owner = calloc(length, sizeof(char));
-	if (owner==NULL)
+	if (owner==NULL) {
+		fprintf(stderr, "UNABLE TO MALLOC SPACE");
 		return NULL;
+	}
 
 	/* read the current owner string four characters at a time */
 
 	string = owner;
 	for (i=0;i<length;i+=4) {
 		reg.eax = HCI_GET;
-		reg.esi = i;
-		if (HciFunction(&reg)!=0x00)
+		reg.ebx = 0x29;
+		reg.esi = (unsigned long) i;
+		printf("index=%d    ", reg.esi);
+		if (HciFunction(&reg)!=0x00) {
+			fprintf(stdout, "ERROR : reading ownerstring\n");
 			break;
+		}
 
 		/* add non NULL characters to the string */
 
-		valid = reg.ecx & 0xff;
+		valid = reg.ecx & 0xffff;
+		printf("index=%d  valid=%d\n", reg.esi, valid);
 		characters = reg.edx;
-		for (j=0;j<4 && j<valid;j++) {
+		for (j=0;j<4;j++) {
 			c = (char) characters & 0xff;
+			printf("%02x ", c);
 			characters = characters >> 8;
-		       if ((c>=0x20) && (c<=0x80))
+		        if ((c>=0x20) && (c<=0x80))
 				*(string++) = c;
 			if (c==0x0d)
 				*(string++) = c;
@@ -133,6 +141,7 @@ int SetOwnerString(char *owner)
 	printf("length=%d\n", length);
 
 	reg.eax = HCI_SET;
+	reg.ebx = HCI_OWNERSTRING;
 	reg.ecx = 4;
 	reg.esi = 0;
 	reg.edx = 0x74736554;
@@ -166,11 +175,11 @@ int main(int argc, char *argv[])
 
 	/* this program *must* be run as root */
 
-	if (getuid()) {
+/*	if (getuid()) {
 		fprintf(stderr, "ownerstring: must be run as root.\n");
 		return 1;
 	}
-
+*/
 
 	/* process command line arguments */
 
@@ -187,24 +196,24 @@ int main(int argc, char *argv[])
 
 	/* if we recieve a signal, exit cleanly */
 
-	signal(SIGHUP, catch);
-	signal(SIGINT, catch);
-	signal(SIGQUIT, catch);
-	signal(SIGILL, catch);
-	signal(SIGTRAP, catch);
-	signal(SIGABRT, catch);
-	signal(SIGIOT, catch);
-	signal(SIGFPE, catch);
-	signal(SIGKILL, catch);
-	signal(SIGSEGV, catch);
-	signal(SIGPIPE, catch);
- 	signal(SIGTERM, catch);
-	signal(SIGCHLD, catch);
-	signal(SIGCONT, catch);
-	signal(SIGSTOP, catch);
-	signal(SIGTSTP, catch);
-	signal(SIGTTIN, catch);
-	signal(SIGTTOU, catch);
+	signal(SIGHUP, Catch);
+	signal(SIGINT, Catch);
+	signal(SIGQUIT, Catch);
+	signal(SIGILL, Catch);
+	signal(SIGTRAP, Catch);
+	signal(SIGABRT, Catch);
+	signal(SIGIOT, Catch);
+	signal(SIGFPE, Catch);
+	signal(SIGKILL, Catch);
+	signal(SIGSEGV, Catch);
+	signal(SIGPIPE, Catch);
+ 	signal(SIGTERM, Catch);
+	signal(SIGCHLD, Catch);
+	signal(SIGCONT, Catch);
+	signal(SIGSTOP, Catch);
+	signal(SIGTSTP, Catch);
+	signal(SIGTTIN, Catch);
+	signal(SIGTTOU, Catch);
 
 	/* check to see if a copy of ownerstring is already running */
 
@@ -242,7 +251,7 @@ int main(int argc, char *argv[])
 	owner = GetOwnerString();
 	printf("%s\n", owner);
 
-	SetOwnerString(new);
+/*	SetOwnerString(new);*/
 
 /*	add_history(new);
 	owner = readline(NULL);
