@@ -1,14 +1,6 @@
 /* sci.c -- System Configuration Interface
  *
- * Copyright (c) 1998/99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
- *
- * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
- *
- *   This code is covered by the GNU GPL and you are free to make any
- *   changes you wish to it under the terms of the license. However the
- *   code has the potential to render your computer and/or someone else's
- *   unuseable. Unless you truely understand what is going on, I urge you
- *   not to make any modifications and use it as it stands.
+ * Copyright (c) 1998-2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *   
  * $Log: sci.c,v $
  * Revision 1.13  1999/12/18 14:32:24  jab
@@ -81,7 +73,6 @@ static const char rcsid[]="$Id: sci.c,v 1.13 1999/12/18 14:32:24 jab Exp jab $";
 #include<sys/ioctl.h>
 
 #include"sci.h"
-#include"toshiba.h"
 
 
 /*
@@ -89,7 +80,7 @@ static const char rcsid[]="$Id: sci.c,v 1.13 1999/12/18 14:32:24 jab Exp jab $";
  */
 int SciSupportCheck(int *version)
 {
-	unsigned long regs[6];
+	SMMRegisters regs;
 	int fd;
 
 	if ((fd=open(TOSH_DEVICE, O_RDWR))<0)
@@ -100,20 +91,20 @@ int SciSupportCheck(int *version)
 		return SCI_FAILURE;
 	}
 
-	regs[0] = 0xf0f0;
-	regs[1] = 0x0000;
-	regs[2] = 0x0000;
-	regs[3] = 0x0000;
+	regs.eax = 0xf0f0;
+	regs.ebx = 0x0000;
+	regs.ecx = 0x0000;
+	regs.edx = 0x0000;
 
-	if (ioctl(fd, TOSH_SMM, regs)<0) {
+	if (ioctl(fd, TOSH_SMM, &regs)<0) {
 		close(fd);
 		return SCI_FAILURE;
 	}
 	close(fd);
 
-	*version = (int) regs[3];
+	*version = (int) regs.edx;
 
-	return (int) (regs[0] & 0xff00)>>8;
+	return (int) (regs.eax & 0xff00)>>8;
 }
 
 
@@ -124,23 +115,23 @@ int SciSupportCheck(int *version)
  */
 int SciOpenInterface(void)
 {
-	unsigned long regs[6];
+	SMMRegisters regs;
 	int fd;
 
 	if ((fd=open(TOSH_DEVICE, O_RDWR ))<0)
 		return SCI_FAILURE;
 
-	regs[0] = 0xf1f1;
-	regs[1] = 0x0000;
-	regs[2] = 0x0000;
+	regs.eax = 0xf1f1;
+	regs.ebx = 0x0000;
+	regs.ecx = 0x0000;
 
-	if (ioctl(fd, TOSH_SMM, regs)<0) {
+	if (ioctl(fd, TOSH_SMM, &regs)<0) {
 		close(fd);
 		return SCI_FAILURE;
 	}
 	close(fd);
 
-	return (int) (regs[0] & 0xff00)>>8;
+	return (int) (regs.eax & 0xff00)>>8;
 }
 
 
@@ -149,43 +140,37 @@ int SciOpenInterface(void)
  */
 int SciCloseInterface(void)
 {
-	unsigned long regs[6];
+	SMMRegisters regs;
 	int fd;
 
 	if ((fd=open(TOSH_DEVICE, O_RDWR ))<0)
 		return SCI_FAILURE;
 
-	regs[0] = 0xf2f2;
-	regs[1] = 0x0000;
-	regs[2] = 0x0000;
+	regs.eax = 0xf2f2;
+	regs.ebx = 0x0000;
+	regs.ecx = 0x0000;
 
-	if (ioctl(fd, TOSH_SMM, regs)<0) {
+	if (ioctl(fd, TOSH_SMM, &regs)<0) {
 		close(fd);
 		return SCI_FAILURE;
 	}
 	close(fd);
 
-	return (int) (regs[0] & 0xff00)>>8;
+	return (int) (regs.eax & 0xff00)>>8;
 }
 
 
 /*
  * Get the setting of a given mode of the laptop
  */
-int SciGet(SMMRegisters *reg)
+int SciGet(SMMRegisters *regs)
 {
-	unsigned long regs[6];
 	int fd;
 
 	if ((fd=open(TOSH_DEVICE, O_RDWR ))<0)
 		return SCI_FAILURE;
 
-	regs[0] = 0xf3f3;
-	regs[1] = reg->ebx;
-	regs[2] = reg->ecx;
-	regs[3] = reg->edx;
-	regs[4] = reg->esi;
-	regs[5] = reg->edi;
+	regs->eax = 0xf3f3;
 
 	if (ioctl(fd, TOSH_SMM, regs)<0) {
 		close(fd);
@@ -193,34 +178,21 @@ int SciGet(SMMRegisters *reg)
 	}
 	close(fd);
 
-	reg->eax = regs[0];
-	reg->ebx = regs[1];
-	reg->ecx = regs[2];
-	reg->edx = regs[3];
-	reg->esi = regs[4];
-	reg->edi = regs[5];
-
-	return (int) (regs[0] & 0xff00)>>8;
+	return (int) (regs->eax & 0xff00)>>8;
 }
 
 
 /*
  * Set the setting of a given mode of the laptop
  */
-int SciSet(SMMRegisters *reg)
+int SciSet(SMMRegisters *regs)
 {
-	unsigned long regs[6];
 	int fd;
 
 	if ((fd=open(TOSH_DEVICE, O_RDWR ))<0)
 		return SCI_FAILURE;
 
-	regs[0] = 0xf4f4;
-	regs[1] = reg->ebx;
-	regs[2] = reg->ecx;
-	regs[3] = reg->edx;
-	regs[4] = reg->esi;
-	regs[5] = reg->edi;
+	regs->eax = 0xf4f4;
 
 	if (ioctl(fd, TOSH_SMM, regs)<0) {
 		close(fd);
@@ -228,14 +200,7 @@ int SciSet(SMMRegisters *reg)
 	}
 	close(fd);
 
-	reg->eax = regs[0];
-	reg->ebx = regs[1];
-	reg->ecx = regs[2];
-	reg->edx = regs[3];
-	reg->esi = regs[4];
-	reg->edi = regs[5];
-
-	return (int) (regs[0] & 0xff00)>>8;
+	return (int) (regs->eax & 0xff00)>>8;
 }
 
 
@@ -244,7 +209,7 @@ int SciSet(SMMRegisters *reg)
  */
 int SciACPower(void)
 {
-	unsigned long regs[6];
+	SMMRegisters regs;
 	int fd;
 
 	if (access(TOSH_PROC, R_OK))
@@ -253,52 +218,16 @@ int SciACPower(void)
 	if ((fd=open(TOSH_DEVICE, O_RDWR))<0)
 		return SCI_FAILURE;
 
-	regs[0] = 0xfefe;
-	regs[1] = 0x0003;
-	regs[2] = 0x0000;
-	regs[3] = 0x0000;
+	regs.eax = 0xfefe;
+	regs.ebx = 0x0003;
+	regs.ecx = 0x0000;
+	regs.edx = 0x0000;
 
-	if (ioctl(fd, TOSH_SMM, regs)<0) {
+	if (ioctl(fd, TOSH_SMM, &regs)<0) {
 		close(fd);
 		return 0;
 	}
 	close(fd);
 
-	return (int) regs[2];
-}
-
-
-/*
- * Set a supervisor or user password for the laptop.
- *
- *   To remove a password, call this routine with the *same* password.
- *   In theory this should be done via the Set routine.
- *
- */
-int SciSetPassword(char *password, unsigned short type, int *trys)
-{
-	unsigned long regs[6];
-	int fd;
-
-	if ((fd=open(TOSH_DEVICE, O_RDWR ))<0)
-		return SCI_FAILURE;
-
-	regs[0] = 0xf4f4;
-	regs[1] = 0x0602;
-	regs[2] = type;
-	regs[3] = *(password+3) + *(password+2)*0x100 + *(password+1)*0x10000 +
-	        *(password)*0x1000000;
-	regs[4] = *(password+7) + *(password+6)*0x100 + *(password+5)*0x10000 +
-	        *(password+4)*0x1000000;
-	regs[5] = *(password+9) + *(password+8)*0x100;
-
-	if (ioctl(fd, TOSH_SMM, regs)<0) {
-		close(fd);
-		return SCI_FAILURE;
-	}
-	close(fd);
-
-	*trys = (int) (regs[3] & 0xffff);
-
-	return (int) (regs[0] & 0xff00)>>8;
+	return (int) regs.ecx;
 }
