@@ -187,7 +187,8 @@ enum {
 
 enum {
 	SCI_BRIGHT          = 0x0000,
-	SCI_SEMI_BRIGHT     = 0x0001
+	SCI_SEMI_BRIGHT     = 0x0001,
+	SCI_SUPER_BRIGHT    = 0x0002
 };
 
 enum {
