@@ -11,6 +11,10 @@
  * <dhinds@pcmcia.sourceforge.org
  *
  * $Log: thotswap.c,v $
+ * Revision 1.3  2000/02/12 11:25:50  jab
+ * switched to using waitpid to prevent possible race condition
+ * fixed bug with not calling exit if execv failed in child processes
+ *
  * Revision 1.2  1999/12/21 12:01:44  jab
  * added beeps to inform the user of errors/success
  *
@@ -34,7 +38,7 @@
  *
  */
 
-static char const rcsid[]="$Id: thotswap.c,v 1.2 1999/12/21 12:01:44 jab Exp jab $";
+static char const rcsid[]="$Id: thotswap.c,v 1.3 2000/02/12 11:25:50 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -74,12 +78,6 @@ static int quite = 0;
 static int console = 1;
 
 
-#define PMAX 5
-static long pid[PMAX];
-static int retstatus[PMAX];
-static int phead = 0;
-static int ptail = 0;
-
 #define BEEP_TIME 150
 #define BEEP_OK   1000
 #define BEEP_WARN 2000
@@ -112,64 +110,29 @@ void CatchSignal(int x)
 }
 
 
-/*
- * Handle SIGCHLD signals
- *
- * This function is called by the kernel when one of our child processes dies.
- * We need to call wait to get the status of the child and save it in a ring
- * buffer so it can be examined later. Before returning, we re-register the
- * signal handler.
- */
-void NoZombie(int sig)
-{
-	long exited = 0;
-	int status = 0;
-
-	exited = wait(&status);
-
-	if ((phead==ptail-1) || (phead==PMAX-1 && ptail==0)) {
-		/* Ahhhhhhh. Fork bomb */
-		return;
-	} else {
-		pid[phead] = exited;
-		retstatus[phead] = status;
-		phead = (phead==PMAX-1) ? 0 : ++phead;
-	}
-
-	signal(SIGCHLD, NoZombie);
-	return;
-}
-
 
 /*
  * Determine if configuration information has changed and needs to be re-read
  */
 void HandleDeadChild(long pid, int status)
 {
-	/* I don't quite understand the values here but it works */
-
-	if (status==25600) {
-		/* reread the alarm settings */
-	}
-
-	if (status==51200) {
-		/* set the battery save options */
-	}
-
 	return;
 }
 
 
 /*
- * This function is called from the main loop. It's job is to empty the
- * ring buffers filled by the SIGCHLD signal handler
+ * This function is called from the main loop. It's job is to find
+ * dead children.
  */
 void ShovelDeadChildren()
 {
-	while (phead!=ptail) {
-		HandleDeadChild(pid[ptail], retstatus[ptail]);
-		ptail = (ptail==PMAX-1) ? 0 : ++ptail;
-	}
+	pid_t childpid;
+	int status;
+
+	while ((childpid = waitpid(-1, &status, WNOHANG))>0) {
+		HandleDeadChild(childpid, status);
+ 	}
+
 	return;
 }
 
@@ -211,7 +174,7 @@ int DisplayXMessage(char *message)
 		close(lead[0]);
 		close(lead[1]);
 		execv(XMESSAGE, argv);
-		return 127;
+		exit(127);
 	}
 
 	return -1;
@@ -263,7 +226,7 @@ int WriteMessage(char *message)
 		close(lead[0]);
 		close(lead[1]);
 		execv(WALL, argv);
-		return 127;
+		exit(127);
 	}
 
 	return -1;
@@ -524,7 +487,6 @@ void Daemon(void)
 	if (signal(SIGINT, SIG_IGN)!=SIG_IGN) signal(SIGINT, CatchSignal);
 	if (signal(SIGQUIT, SIG_IGN )!=SIG_IGN) signal(SIGQUIT, CatchSignal);
 	if (signal(SIGTERM, SIG_IGN)!=SIG_IGN) signal(SIGTERM, CatchSignal);
-	if (signal(SIGCHLD, SIG_IGN)!=SIG_IGN) signal(SIGCHLD, NoZombie);
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGUSR1, SIG_IGN);
 
