@@ -8,6 +8,15 @@
  *       mount.app by Steve Borho <borho@stlnet.com>
  *
  * $Log: wmtuxtime.c,v $
+ * Revision 1.6  2000/02/12 21:33:31  jab
+ * added patch from Stephen Morphet <smorphet@iee.org> to use a fallback to
+ * /proc/apm for time remaining when SCI_BATTERY_TIME does not work
+ *
+ * Revision 1.5  2000/02/12 10:55:55  jab
+ * patchs to use waitpid to prevent possible race conditions, update the
+ * display less franetically and get the CPU speed working courtesy
+ * of Tom May <tom@you-bastards.com>
+ *
  * Revision 1.4  2000/01/21 22:16:25  jab
  * fixed bug with blank xmessage windows when launched from dock
  *
@@ -37,7 +46,7 @@
  *
  */
 
-static char const rcsid[]="$Id: wmtuxtime.c,v 1.4 2000/01/21 22:16:25 jab Exp jab $";
+static char const rcsid[]="$Id: wmtuxtime.c,v 1.6 2000/02/12 21:33:31 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -69,6 +78,7 @@ static char const rcsid[]="$Id: wmtuxtime.c,v 1.4 2000/01/21 22:16:25 jab Exp ja
 
 
 enum { EMPTY, PRIMARY, AUXILLARY };
+enum { APM_READ, APM_CLEAR };
 
 static int id;
 static int power = SCI_MAINS;
@@ -78,11 +88,16 @@ static int alarms;
 static char *config;
 static BatteryState low,critical;
 
-#define PMAX 5
-static long pid[PMAX];
-static int retstatus[PMAX];
-static int phead = 0;
-static int ptail = 0;
+/* Structure containing data read from /proc/apm */
+static struct apm_data_t
+{
+	/* Status */
+	int valid;
+	int read_failed;
+	/* Data */
+	int battery;
+	int time;
+} apm_data;
 
 
 /*
@@ -98,35 +113,6 @@ void CatchSignal(int x)
 }
 
 
-/*
- * Handle SIGCHLD signals
- *
- * This function is called by the kernel when one of our child processes dies.
- * We need to call wait to get the status of the child and save it in a ring
- * buffer so it can be examined later. Before returning, we re-register the
- * signal handler.
- */
-void NoZombie(int sig)
-{
-	long exited = 0;
-	int status = 0;
-
-	exited = wait(&status);
-
-	if ((phead==ptail-1) || (phead==PMAX-1 && ptail==0)) {
-		/* Ahhhhhhh. Fork bomb */
-		return;
-	} else {
-		pid[phead] = exited;
-		retstatus[phead] = status;
-		phead = (phead==PMAX-1) ? 0 : ++phead;
-	}
-
-	signal(SIGCHLD, NoZombie);
-	return;
-}
-
-
 /*
  * Install signal handlers so we can clean up nicely on an error
  */
@@ -147,12 +133,34 @@ void HandleSignals(void)
 	signal(SIGSTOP, CatchSignal);
 	signal(SIGTTIN, CatchSignal);
 	signal(SIGTTOU, CatchSignal);
-	signal(SIGCHLD, NoZombie);
 	signal(SIGUSR1, SIG_IGN);
 
 	return;
 }
 
+/*
+ * Process the command line arguments
+ */
+void ProcessComandLine(int *argc, char ***argv)
+{
+	int i,j;
+
+	for (i=1;i<*argc;i++) {
+		if ((!strcmp((*argv)[i], "-c")) || (!strcmp((*argv)[i], "--contrast"))) {
+/*			for (j=5;j<12;j++)
+				wmtuxtime_xpm[3][j] = '0';
+			for (j=5;j<12;j++)
+				wmtuxtime_xpm[4][j] = '0';
+			for (j=5;j<12;j++)
+				wmtuxtime_xpm[7][j] = '0';
+			for (j=5;j<12;j++)
+				wmtuxtime_xpm[9][j] = '0';*/
+		}
+	}
+
+	return;
+}
+
 
 /*
  * Use xmessage to display the battery alert messages
@@ -186,7 +194,7 @@ int DisplayMessage(char *message, char *title)
 		close(lead[0]);
 		close(lead[1]);
 		execv(XMESSAGE, argv);
-		return 127;
+		exit(127);
 	}
 
 	return -1;
@@ -202,7 +210,7 @@ int CallConfigApp(char *settings)
 		char *argv[3] = {"tuxtime-conf", settings, NULL};
 
 		execv(BINDIR"/tuxtime-conf", argv);
-		return 127;
+		exit(127);
 	}
 
 	return -1;
@@ -239,8 +247,8 @@ void PressEvent(XButtonEvent *event)
 	if(x>=6 && y>=48 && x<=18 && y<=58) {
 		/* next arrow button */
 
-	 	copyXPMArea(66, 44, 13, 11, 6, 48);
 		if ((selectbay!=EMPTY) && (power==SCI_BATTERY)) {
+		 	copyXPMArea(66, 44, 13, 11, 6, 48);
 			switch (selectbay) {
 				case PRIMARY:
 					copyXPMArea(79, 93, 15, 10, 44, 33);
@@ -255,19 +263,17 @@ void PressEvent(XButtonEvent *event)
 					selectbay = PRIMARY;
 					break;
 			}
-		} else {
-			XBell(display, 100);
 		}
 	} else if (x>=19 && y>=48 && x<=31 && y<=58){
-		/* alarm settings */
-
-		copyXPMArea(79, 44, 13, 11, 19, 48);
-		CallConfigApp("--alarms");
-	} else if (x>=32 && y>=48 && x<=44 && y<=58){
 		/* suspend call */
 
-		copyXPMArea(92, 44, 13, 11, 32, 48);
+		copyXPMArea(79, 44, 13, 11, 19, 48);
 		SoftwareSuspend();
+	} else if (x>=32 && y>=48 && x<=44 && y<=58){
+		/* alarm settings */
+
+		copyXPMArea(92, 44, 13, 11, 32, 48);
+		CallConfigApp("--alarms");
 	} else if (x>=45 && y>=48 && x<=57 && y<=58){
 		/* machine settings */
 
@@ -275,6 +281,8 @@ void PressEvent(XButtonEvent *event)
 		CallConfigApp("--settings");
 	}
 
+	RedrawWindow();
+
 	return;
 }
 
@@ -536,6 +544,21 @@ void PowerChanged(int source, char *file)
 				"method\n");
 		}
 
+		/* toggle battery save mode to ensure settings take effect */
+
+		reg.ebx = SCI_BATTERY_SAVE;
+		reg.ecx = 1;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set battery "
+				"save mode (1)\n");
+		}
+
+		reg.ebx = SCI_BATTERY_SAVE;
+		reg.ecx = save;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set battery "
+				"save mode (save)\n");
+		}
 	}
 
 	SciCloseInterface();
@@ -592,45 +615,93 @@ void DisplayAlarm(int state, int percent, int time)
 }
 
 
+/*
+ * Get information from /proc/apm if avaliable.
+ */
+void ReadApm(int clear)
+{
+	FILE *str;
+	char version[10],buffer[64];
+
+	if (clear==APM_CLEAR) {
+		/* invalidate the stored data */
+		apm_data.valid = 0;
+		return;
+	}
+
+	/* If /proc/apm read has failed in the past, let's assume it
+	   will fail again.  Otherwise, only read if it has not been
+	   read since the last clear. */
+
+	if((!apm_data.read_failed) && (!apm_data.valid)) {
+		if (access("/proc/apm", R_OK)) {
+			/* cannot find /proc/apm */
+			fprintf(stderr, 
+				"wmtuxtime: unable to access /proc/apm\n");
+			apm_data.read_failed = 1;
+			return;
+		} else {
+			/* open /proc/apm for reading */
+			if (!(str = fopen("/proc/apm", "r"))) {
+				fprintf(stderr, "wmtuxtime: "
+					"unable to open /proc/apm\n");
+				apm_data.read_failed = 1;
+				return;
+			}
+
+			/* scan in the information */
+			fgets(buffer, sizeof(buffer)-1, str);
+			fclose(str);
+			buffer[sizeof(buffer)-1] = '\0';
+
+			sscanf(buffer, 
+				"%s %*d.%*d %*x %*x %x %*x %*d%% %d %*s\n",
+				version, &(apm_data.battery), &(apm_data.time));
+	  
+			/* if old style driver return */
+			if (version[0]=='B') {
+				fprintf(stderr, 
+					"wmtuxtime: old style apm driver\n");
+				apm_data.read_failed = 1;
+				return;
+			}
+
+			apm_data.valid = 1;
+		}
+	}
+
+	return;
+}
+
+
 /*
  * Determine if the battery is charging by reading /proc/apm if avaliable
  */
 int BatteryCharging(void)
 {
-	FILE *str;
-	int battery;
-	char version[10],buffer[64];
+	ReadApm( APM_READ );
+	return apm_data.battery;
+}
 
 
-	if (access("/proc/apm", R_OK)) {
-		/* cannot find /proc/apm */
-		fprintf(stderr, "wmtuxtime: unable to access /proc/apm\n");
-		return 0;
-	} else {
+/*
+ * Fallback routine for when SCI_BATTERY_TIME fails to use /proc/apm
+ */
+int TimeRemaining(void)
+{
+	int hours = 0;
+	int minutes;
 
-		/* open /proc/apm for reading */
-		if (!(str = fopen("/proc/apm", "r"))) {
-			fprintf(stderr, "wmtuxtime: unable to open /proc/apm\n");
-			return 0;
-		}
+	ReadApm(APM_READ);
 
-		/* scan in the information */
-		fgets(buffer, sizeof(buffer)-1, str);
-		fclose(str);
-		buffer[sizeof(buffer)-1] = '\0';
-		sscanf(buffer, "%s %*d.%*d %*x %*x %x %*x %*d%% %*d %*s\n",
-			version, &battery);
+	if(apm_data.valid) {
+		minutes = apm_data.time;
+		hours = minutes/60;
+		minutes -= (60*hours);
+		return SCI_TIME(hours,minutes);
+        }
 
-		/* if old style driver return */
-		if (version[0]=='B') {
-			fprintf(stderr, "wmtuxtime: old style apm driver\n");
-			return 0;
-		}
-
-		return battery;
-	}
-
-	return 0;
+	return -1;
 }
 
 
@@ -643,10 +714,15 @@ void Update(void)
 	SMMRegisters reg;
 	int time,percent,second,battery;
 	int hour,minute,state,i,j;
+	int time_is_apm;
 
 	time = 0;
+	time_is_apm = 0;
 	percent = 0;
 
+	/* clear the apm data, it will be read again next time it's needed. */
+	ReadApm(APM_CLEAR);
+
 	/* get the percent and time remaining */
 
 	SciOpenInterface();
@@ -666,10 +742,14 @@ void Update(void)
 	}
 
 	reg.ebx = SCI_BATTERY_TIME;
-	if (SciGet(&reg)==SCI_SUCCESS)
+	if (SciGet(&reg)==SCI_SUCCESS) {
 		time = (reg.ecx & 0xffff);
-	else
-		time = -1;
+	} else {
+		/* fall back to reading /proc/apm. */
+
+		time = TimeRemaining();
+		time_is_apm = 1;
+	}
 
 	/* check to see if their is a battery in the SelectBay */
 
@@ -681,11 +761,20 @@ void Update(void)
 			second = -1;
 		if ((power==SCI_BATTERY) && (selectbay==EMPTY))
 			copyXPMArea(62, 93, 15, 10, 44, 33);
-		if (selectbay==EMPTY)
+		if (selectbay==EMPTY) {
 			selectbay = PRIMARY;
+			/* activate the arrow button */
+			copyXPMArea(119, 25, 4, 7,  11, 50);
+			copyXPMArea(119, 25, 4, 7,  71, 58);
+			copyXPMArea(119, 25, 4, 7,  71, 46);
+		}
 	} else {
 		selectbay = EMPTY;
 		second = -1;
+		/* grey out the arrow button */
+		copyXPMArea(119, 35, 4, 7,  11, 50);
+		copyXPMArea(119, 35, 4, 7,  71, 58);
+		copyXPMArea(119, 35, 4, 7,  71, 46);
 	}
 
 	SciCloseInterface();
@@ -764,7 +853,7 @@ void Update(void)
 	     If not charging: Time left before battery drains to 0% 
 	     If charging:     Time left before battery gets to maximum */
 
-	if (time<0) {
+	if (time<0 || ((time_is_apm) && (power==SCI_MAINS))) {
 		/* dodgy time so just display a "null" indicator (--:--) */
 
 		copyXPMArea(83, 69, 41, 9, 15, 7);
@@ -826,7 +915,6 @@ void Update(void)
 
 		copyXPMArea(66, j, i, 9, 7, 21);
 	}
-
 		
 	return;
 }
@@ -835,17 +923,15 @@ void Update(void)
 /*
  * Determine if configuration information has changed and needs to be re-read
  */
-void HandleDeadChild(long pid, int status)
+void HandleDeadChild(pid_t pid, int status)
 {
-	/* I don't quite understand the values here but it works */
-
-	if (status==25600) {
+	if (WIFEXITED(status) && WEXITSTATUS(status)==0x10) {
 		/* reread the alarm settings */
 		SetupAlarms(config);
 		Update();
 	}
 
-	if (status==51200) {
+	if (WIFEXITED(status) && WEXITSTATUS(status)==0x20) {
 		/* set the battery save options */
 		PowerChanged(power, config);
 	}
@@ -855,15 +941,17 @@ void HandleDeadChild(long pid, int status)
 
 
 /*
- * This function is called from the main loop. It's job is to empty the
- * ring buffers filled by the SIGCHLD signal handler
+ * This function is called from the main loop. It's job is to find
+ * dead children.
  */
 void ShovelDeadChildren()
 {
-	while (phead!=ptail) {
-		HandleDeadChild(pid[ptail], retstatus[ptail]);
-		ptail = (ptail==PMAX-1) ? 0 : ++ptail;
-	}
+	pid_t childpid;
+	int status;
+
+	while ((childpid = waitpid(-1, &status, WNOHANG))>0) {
+		HandleDeadChild(childpid, status);
+ 	}
 
 	return;
 }
@@ -907,7 +995,8 @@ int main(int argc, char *argv[])
         setegid(getgid());
 
 	switch (id) {
-		case 0xfc09: /* Satellite 2610CDT */
+		case 0xfc09: /* Satellite 2610CDT, 2650XDVD */
+		case 0xfc0c: /* Tecra 8100x */
 		case 0xfc0f: /* Satellite 2060CDS/CDT */
 		case 0xfc10: /* Satellite 2550/2590 */
 		case 0xfc11: /* Portage 3110CT */
@@ -1043,9 +1132,8 @@ int main(int argc, char *argv[])
 		CreateDefaultRC(config);
 	}
 
-	/* install signal handling */
-
 	HandleSignals();
+	ProcessComandLine(&argc, &argv);
 
 	/* read the config file and initilize state */
 
@@ -1057,11 +1145,17 @@ int main(int argc, char *argv[])
 		power = SCI_MAINS;
 	poll = GetConfigInt("TuxTime", "PollInterval", 10, config);
 
+
 	/* open the X11 windows */
 
 	openXwindow(argc, argv, wmtuxtime_xpm, wmtuxtime_mask_bits,
 		wmtuxtime_mask_width, wmtuxtime_mask_height);
 
+	/* update immediately to make sure it's right */
+
+	Update();
+	RedrawWindow();
+
 	/* loop until we die... */
 
 	i = 0;
@@ -1070,11 +1164,14 @@ int main(int argc, char *argv[])
 		/*  only process battery state every poll cycles of this loop.
 		    We run it faster to catch the expose events, etc... */ 
 
-		if (i>poll) {
+		if (i++>poll) {
 			Update();
+			RedrawWindow();
 			i = 0;
-		} else {
-			i++;
+
+			/* handle dead child processes */
+
+			ShovelDeadChildren();
 		}
 
 		/* process any pending X events */
@@ -1094,13 +1191,9 @@ int main(int argc, char *argv[])
 			}
 		}
 
-		/* handle dead child processes */
 
-		ShovelDeadChildren();
+		/* wait for next update  */
 
-		/* Redraw and wait for next update  */
-
-		RedrawWindow();
 		usleep(DELAY);
 	}
 
